@@ -7,6 +7,7 @@
 #'
 #' @param dat either data.frame or data.table containing the survey data with
 #'   rotating panel design.
+#' @param method for bootstrap replicates, either `"Preston"` or `"Rao-Wu"` 
 #' @param REP integer indicating the number of bootstrap replicates.
 #' @param hid character specifying the name of the column in `dat` containing
 #'   the household id. If `NULL` (the default), the household structure is not
@@ -48,12 +49,12 @@
 #'   for more information see Details.
 #' @param pid column in `dat` specifying the personal identifier. This
 #'   identifier needs to be unique for each person throught the whole data set.
-#' @param new.method logical, if TRUE bootstrap replicates will never be
-#'   negative even if in some strata the whole population is in the sample.
-#'   WARNING: This is still experimental and resulting standard errors might be
-#'   underestimated! Use this if for some strata the whole population is in the
-#'   sample!
-#'
+#'   Can be `NULL`.
+#' @param seed integer specifying the seed for the random number generator.
+#' @param already.selected list of data.tables indicating if record was drawn
+#' in previous `period`. `length(already.selected)` must be equal to 
+#' the number of sampling stages specified. See [get.selection()] for an example. 
+#' 
 #' @return the survey data with the number of REP bootstrap replicates added as
 #'   columns.
 #'
@@ -71,49 +72,56 @@
 #' * Columns by which population was stratified during the sampling process
 #'   (parameter: `strata`).
 #'
+#' As methods either the either the rescaled bootstrap for stratified 
+#' multistage sampling, presented by J. Preston (2009) (`method = "Preston"`) 
+#' or the Rao-Wu boostrap by J. N. K. Rao and C. F. J. Wu (1988) (`method = "Rao-Wu"`)
+#' are supported.
+#' \cr
 #' For single stage sampling design a column the argument `totals` is optional,
 #' meaning that a column of the number of PSUs at the first stage does not need
-#' to be supplied.
-#' For this case the number of PSUs is calculated and added to `dat` using
+#' to be supplied. The number of PSUs is calculated and added to `dat` using
 #' `strata` and `weights`. By setting `cluster` to NULL single stage sampling
 #' design is always assumed and
 #' if `strata` contains of multiple column names the combination of all those
 #' column names will be used for stratification.
-#'
+#' \cr
 #' In the case of multi stage sampling design the argument `totals` needs to be
 #' specified and needs to have the same number of arguments as `strata`.
-#'
+#' \cr
 #' If `cluster` is `NULL` or does not contain `hid` at the last stage, `hid`
 #' will automatically be used as the final cluster. If, besides `hid`,
 #' clustering in additional stages is specified the number of column names in
 #' `strata` and `cluster` (including `hid`) must be the same. If for any stage
 #' there was no clustering or stratification one can set "1" or "I" for this
 #' stage.
-#'
+#' \cr
 #' For example `strata=c("REGION","I"),cluster=c("MUNICIPALITY","HID")` would
 #' speficy a 2 stage sampling design where at the first stage the municipalities
 #' where drawn stratified by regions
 #' and at the 2nd stage housholds are drawn in each municipality without
 #' stratification.
-#'
-#' Bootstrap replicates are drawn for each survey period (`period`) using the
+#' \cr
+#' Bootstrap replicates are drawn for each survey period consecutively (`period`) using the
 #' function [rescaled.bootstrap].
-#' Afterwards the bootstrap replicates for each household are carried forward
-#' from the first period the household enters the survey to all the censecutive
-#' periods it stays in the survey.
-#'
+#' Bootstrap replicates are drawn consistently in the way that in each `period` and
+#' sampling stage always \eqn{\lfloor n/2 \rfloor} clusters are selected in each strata.
+#' \cr
 #' This ensures that the bootstrap replicates follow the same logic as the
 #' sampled households, making the bootstrap replicates more comparable to the
 #' actual sample units.
-#'
+#' \cr
 #' If `split` ist set to `TRUE` and `pid` is specified, the bootstrap replicates
-#' are carried forward using the personal identifiers instead of the houshold
+#' are carried forward using the personal identifiers instead of the household
 #' identifier.
 #' This takes into account the issue of a houshold splitting up.
 #' Any person in this new split household will get the same bootstrap replicate
 #' as the person that has come from an other household in the survey.
 #' People who enter already existing households will also get the same bootstrap
 #' replicate as the other households members had in the previous periods.
+#' \cr
+#' `already.selected` can be specified in order to construct bootstrap replicates
+#' considering that already drawn bootstrap replicates from a previous `period`
+#' exist for the same survey. See [get.selection()] for more explanations and examples.
 #'
 #' @return Returns a data.table containing the original data as well as the
 #'   number of `REP` columns containing the bootstrap replicates for each
@@ -126,25 +134,34 @@
 #' @seealso [`data.table`][data.table::data.table] for more information on
 #'   data.table objects.
 #'
+#' @references 
+#' Preston, J. (2009). Rescaled bootstrap for stratified multistage
+#'   sampling. Survey Methodology. 35. 227-234.
+#' Rao, J. N. K., and C. F. J. Wu. (1988). Resampling Inference with Complex Survey Data.
+#'  Journal of the American Statistical Association 83 (401): 231–41.
 #' @author Johannes Gussenbauer, Alexander Kowarik, Statistics Austria
 #'
 #' @examples
-#' \dontrun{
-#' eusilc <- demo.eusilc(prettyNames = TRUE)
+#' 
+#' library(surveysd)
+#' library(data.table)
+#' setDTthreads(1)
+#' set.seed(1234)
+#' eusilc <- demo.eusilc(n = 3, prettyNames = TRUE)
 #'
-#' ## draw sample without stratification or clustering
-#' dat_boot <- draw.bootstrap(eusilc, REP = 10, weights = "pWeight",
+#' ## draw replicates without stratification or clustering
+#' dat_boot <- draw.bootstrap(eusilc, REP = 1, weights = "pWeight",
 #'                            period = "year")
 #'
 #' ## use stratification w.r.t. region and clustering w.r.t. households
 #' dat_boot <- draw.bootstrap(
-#'   eusilc, REP = 10, hid = "hid", weights = "pWeight",
+#'   eusilc, REP = 1, hid = "hid", weights = "pWeight",
 #'   strata = "region", period = "year")
 #'
 #' ## use multi-level clustering
 #' dat_boot <- draw.bootstrap(
-#'   eusilc, REP = 10, hid = "hid", weights = "pWeight",
-#'   strata = c("region", "age"), period = "year")
+#'   eusilc, REP = 1, hid = "hid", weights = "pWeight",
+#'   strata = c("region", "hsize"), period = "year")
 #'
 #'
 #' # create spit households
@@ -172,26 +189,31 @@
 #' }
 #'
 #' dat_boot <- draw.bootstrap(
-#'   eusilc, REP = 10, hid = "hid", weights = "pWeight",
-#'   strata = c("region", "age"), period = "year", split = TRUE,
+#'   eusilc, REP = 1, hid = "hid", weights = "pWeight",
+#'   strata = c("region", "hsize"), period = "year", split = TRUE,
 #'   pid = "pidsplit")
 #' # split households were considered e.g. household and
 #' # split household were both selected or not selected
 #' dat_boot[, data.table::uniqueN(w1), by = pidsplit][V1 > 1]
-#' }
+#' 
 #'
 #' @export draw.bootstrap
 #'
 
 
+
 draw.bootstrap <- function(
-  dat, REP = 1000, hid = NULL, weights, period = NULL, strata = NULL,
-  cluster = NULL, totals = NULL, single.PSU = c("merge", "mean"), boot.names =
-    NULL, split = FALSE, pid = NULL, new.method = FALSE) {
-
-  occurence_first_period <- NULL
-
-  ##########################################################
+    dat, method = "Preston", REP = 1000, hid = NULL, weights, period = NULL, strata = NULL,
+    cluster = NULL, totals = NULL, single.PSU = c("merge", "mean"), boot.names =
+      NULL, split = FALSE, pid = NULL, seed = NULL, already.selected = NULL) {
+  
+  occurence_first_period <- V1 <- strata_i <- . <-
+    hid_survey_segment_help <- NULL
+  
+  if (method == "Rao-Wu") {
+    warning("The 'Rao-Wu' method should only be used when the first stage sampling is conducted without replacement. Ensure that your sampling design follows this requirement.")
+  }
+  
   # INPUT CHECKING
   if (is.data.frame(dat)) {
     dat <- as.data.table(dat)
@@ -203,16 +225,16 @@ draw.bootstrap <- function(
   c.names <- colnames(dat)
 
   removeCols <- c()
+
+  # check seed
+  if(!is.null(seed)){
+    check.input(seed, input.name = "seed", input.length=1, input.type="numeric")
+    set.seed(seed)
+  }
+
   # check REP
-  if (length(REP) != 1) {
-    stop("REP must have length 1")
-  }
-  if (!is.numeric(REP)) {
-    stop("REP must contain one numeric value")
-  }
-  if (REP %% 1 != 0) {
-    stop("REP cannot have a decimal part")
-  }
+  check.input(REP, input.name = "REP", input.length=1, input.type="numeric",
+              decimal.possible = FALSE)
 
   # check hid
   hidNULL <- is.null(hid)
@@ -221,24 +243,23 @@ draw.bootstrap <- function(
     dat[, c(hid) := 1:.N]
     removeCols <- c(removeCols, hid)
   }
+  check.input(hid, input.name = "hid", input.length=1, input.type="character",
+              c.names = c.names)
 
-  if (length(hid) != 1) {
-    stop("hid must have length 1")
-  }
-  if (!hid %in% c.names) {
-    stop(paste0(hid, " is not a column in dat"))
+  # check pid
+  pidNULL <- is.null(pid)
+  if (pidNULL) {
+    pid <- generateRandomName(20, colnames(dat))
+    dat[, c(pid) := paste(hid,1:.N, sep="."), by=c(hid)]
+    removeCols <- c(removeCols, pid)
   }
+  check.input(pid, input.name = "pid", input.length=1, input.type="character",
+              c.names = c.names)
 
   # check weights
-  if (length(weights) != 1) {
-    stop("weights must have length 1")
-  }
-  if (!weights %in% c.names) {
-    stop(paste0(weights, " is not a column in dat"))
-  }
-  if (!is.numeric(dt.eval("dat[,", weights, "]"))) {
-    stop(paste0(weights, " must be a numeric column"))
-  }
+  check.input(weights, input.name = "weights", input.length=1, input.type="character",
+              c.names = c.names, dat = dat, dat.column.type = "numeric")
+
 
   # check period
   periodNULL <- is.null(period)
@@ -248,31 +269,33 @@ draw.bootstrap <- function(
     removeCols <- c(removeCols, period)
   }
 
-  if (length(period) != 1) {
-    stop("period must have length 1")
-  }
-  if (!period %in% c.names) {
-    stop(paste0(period, " is not a column in dat"))
-  }
-  if (!is.numeric(dat[[period]]) & !is.integer(dat[[period]])) {
-    stop(paste0(period, " is not an integer or numeric column"))
-  }
+  check.input(period, input.name = "period", input.length=1, input.type="character",
+              c.names = c.names, dat = dat, dat.column.type = "numeric")
 
   # check design
-  if (is.null(strata)) {
-    strata <- "I"
+  strataNULL <- is.null(strata)
+  if (strataNULL) {
+    strata <- generateRandomName(20, colnames(dat))
+    dat[, (strata) := 1]
+    removeCols <- c(removeCols, strata)
   }
 
+  clusterNULL <- FALSE
   if (is.null(cluster)) {
     cluster <- hid
+    clusterNULL <- TRUE
   } else {
     if (length(cluster) == 1) {
       if (cluster %in% c("1", "I")) {
         cluster <- hid
       }
     }
-    if (!hid %in% cluster) {
-      cluster <- c(cluster, hid)
+    if (!hid %in% cluster & !pid %in% cluster) {
+      if(pidNULL==FALSE){
+        cluster <- c(cluster, pid)
+      }else{
+        cluster <- c(cluster, hid)
+      }
     }
   }
 
@@ -312,7 +335,7 @@ draw.bootstrap <- function(
              " sampling design\n none of them can be '1' or 'I'.")
       }
       strata_var_help <- generateRandomName(20, colnames(dat))
-      dat[, c(strata_var_help) := do.call(paste, c(.SD, sep = "-")), ]
+      dat[, c(strata_var_help) := do.call(paste, c(.SD, sep = "-")), .SDcols=c(strata)]
       strata <- strata_var_help
       removeCols <- c(removeCols, strata)
     }
@@ -325,14 +348,14 @@ draw.bootstrap <- function(
             " replicates for single PSUs cases will be missing!")
     single.PSU <- FALSE
   }
-
+  
   # check boot.names
   if (!is.null(boot.names)) {
     if (!grepl("^[[:alpha:]]", boot.names)) {
       stop("boot.names must start with an alphabetic character")
     }
   }
-
+  
   # check split and pid
   if (is.null(split)) {
     stop("split needs to be logical")
@@ -341,47 +364,36 @@ draw.bootstrap <- function(
     stop("split needs to be logical")
   }
   if (split) {
-    if (!is.character(pid)) {
-      stop("when split is TRUE pid needs to be a string")
-    } else {
-      if (length(pid) > 1) {
-        stop("pid can only have length 1")
-      } else {
-        if (!pid %in% c.names) {
-          stop(pid, "is not a column of dat")
-        }
-      }
-    }
+    if (!is.character(pid) | pidNULL == TRUE) {
+      stop("when split is TRUE pid must be specified")
+    } 
     # check if pid is unique in each household and period
-    unique.pid <- dt.eval("dat[,uniqueN(", pid,
-                          ")==.N,by=c(period,hid)][V1==FALSE]")
+    unique.pid <- dat[,uniqueN(pid)==.N, by=c(period, hid),
+                      env = list(pid = pid)]
+    unique.pid <- unique.pid[V1==FALSE]
     if (nrow(unique.pid) > 0) {
       stop("pid is not unique in each household for each period")
     }
   }
-
-  # check totals
-  # if clusters are specified the finite population correction factors must
-  #   be user specified
-  # check input for totals
-  # if no totals are specified then leave them NULL
+  
   if (is.null(totals)) {
     if (length(cluster) == 1) {
-      # if no clusters are specified calculate number of households in each
-      #   strata
+      
       totals <- generateRandomName(20, existingNames = colnames(dat))
       fpc.strata <- strata[!strata %in% c("I", "1")] # nolint
-      dt.eval("dat[,", totals, ":=sum(", weights, "[!duplicated(",
-              hid, ")]),by=c(fpc.strata,period)]")
+      dat[,c(totals) := sum(weights[!duplicated(hid)]),
+          by = c(fpc.strata, period),
+          env = list(weights = weights,
+                     hid = hid)]
       removeCols <- c(removeCols, totals)
     } else {
-
+      
       stop("For multistage sampling the number of PSUs at each level needs to ",
            "be specified!")
     }
-
+    
   } else {
-
+    
     if (length(totals) != length(strata)) {
       stop("totals must be specified for each stage")
     }
@@ -396,79 +408,111 @@ draw.bootstrap <- function(
   # check for each stage that PSUs are not in mutiple strata
   for(i in seq_along(strata)){
     if(!strata[i]%in%c("1","I") & !cluster[i]%in%c("1","I")){
-      countMultiple <- dt.eval("dat[,uniqueN(",strata[i],"),by=c(cluster[i],period)][V1>1]")
+      countMultiple <- dat[,uniqueN(strata_i), by=c(cluster[i], period),
+                           env = list(strata_i = strata[i])]
+      countMultiple <- countMultiple[V1>1]
       if(nrow(countMultiple)>0){
         stop("Some sampling units in ",cluster[i]," occur in multiple strata of ",strata[i])
       }
     }
   }
   
-  ##########################################################
-
   # define sample design
-  strata <- paste(strata, collapse = ">")
-  cluster <- paste(cluster, collapse = ">")
-  totals <- paste(totals, collapse = ">")
-
+  strata_design <- paste(strata, collapse = ">")
+  cluster_design <- paste(cluster, collapse = ">")
+  totals_design <- paste(totals, collapse = ">")
+  
   if (is.null(boot.names)) {
     w.names <- paste0("w", 1:REP)
   } else{
     w.names <- paste0(boot.names, 1:REP)
   }
-
+  
   # calculate bootstrap replicates
-  dat[, c(w.names) := rescaled.bootstrap(
-    dat = copy(.SD), REP = REP, strata = strata, cluster = cluster,
-    fpc = totals, single.PSU = single.PSU, return.value = "replicates",
-    check.input = FALSE, new.method = new.method), by = c(period)]
+  periods <- sort(unique(dat[[period]]))
+  dat_selection_prev <- NULL
+  if(!is.null(already.selected)){
+    dat_selection_prev <- already.selected
+  }
+  for(p in periods){
 
+      dat_boot <- rescaled.bootstrap(dat[.(p),on=c(period)],
+                                   method = method, 
+                                   REP = REP, strata = strata_design, cluster = cluster_design,
+                                   fpc = totals_design, single.PSU = single.PSU, return.value = c("replicates","selection"),
+                                   already.selected = dat_selection_prev,
+                                   run.input.checks = FALSE)
+    dat[period == p, c(w.names) := dat_boot$replicates, env = list(period = period)]
+    dat_selection_prev <- dat_boot$selection
+    rm(dat_boot)
+  }
+  
   # respect split households
   if (split) {
     dat <- generate.HHID(dat, period = period, pid = pid, hid = hid)
-  }
-
-  # keep bootstrap replicates of first period for each household
-  # if households drops out of survey and re-enters survey at a later stage
-  # -> treat as different instances
-  help_survey_grouping <- function(x){
-    x_holes <- which(diff(x)>1)
-    x_groups <- diff(c(0,x_holes,length(x)))
-    x_groups <- rep(1:length(x_groups),times=x_groups)
-    return(x_groups)
-  }
-  dt.eval("dat[,hid_survey_segment_help:=help_survey_grouping(", period, "),by=c(hid)]")
-  dt.eval("dat[,occurence_first_period :=min(", period, "),by=c(hid,'hid_survey_segment_help')]")
-  select.first.occurence <- paste0(c(hid,"hid_survey_segment_help", w.names), collapse = ",")
-  dat.first.occurence <- unique(
-    dt.eval("dat[", period, "==occurence_first_period,.(",
-            select.first.occurence, ")]"
-    ), by = c(hid,"hid_survey_segment_help"))
-  dat[, c(w.names) := NULL]
-  dat <- merge(dat, dat.first.occurence, by = c(hid, "hid_survey_segment_help"), all.x = TRUE)
-  dat[, c("occurence_first_period","hid_survey_segment_help") := NULL]
-
-
-  # remove columns
-  if (split) {
-    dt.eval("dat[,", hid, ":=", paste0(hid, "_orig"), "]")
+    
+    help_survey_grouping <- function(x){
+      x_holes <- which(diff(x)>1)
+      x_groups <- diff(c(0,x_holes,length(x)))
+      x_groups <- rep(1:length(x_groups),times=x_groups)
+      return(x_groups)
+    }
+    dat[,hid_survey_segment_help := help_survey_grouping(period), by=c(hid),
+        env = list(period = period)]
+    dat[,occurence_first_period := min(period), by=c(hid,"hid_survey_segment_help"),
+        env = list(period = period)]
+    # dat[, hid_survey_segment_help := help_survey_grouping(get(period)), by = hid]
+    # dat[, occurence_first_period := min(get(period)), 
+    #     by = .(hid, hid_survey_segment_help)]
+    
+    select.first.occurence <- paste0(c(hid,"hid_survey_segment_help", w.names), collapse = ",")
+    dat.first.occurence <- unique(dat[period==occurence_first_period, .SD,
+                                      .SDcols = c(hid,"hid_survey_segment_help", w.names),
+                                      env = list(period = period)])
+    # select_cols <- c(hid, "hid_survey_segment_help", w.names)
+    # dat.first.occurence <- unique(
+    #   dat[get(period) == occurence_first_period, 
+    #       ..select_cols] 
+    # )
+    
+    dat[, c(w.names) := NULL]
+    dat <- merge(dat, dat.first.occurence, by = c(hid, "hid_survey_segment_help"), all.x = TRUE)
+    dat[, c("occurence_first_period","hid_survey_segment_help") := NULL]
+    dat[, hid := paste0(hid,"_orig"), env = list(hid = hid)]
+    # dat[, (hid) := paste0(get(hid), "_orig")]
     dat[, c(paste0(hid, "_orig")) := NULL]
   }
+  
+  
   if (length(removeCols) > 0) {
     dat[, c(removeCols) := NULL]
   }
 
-
   if (periodNULL) {
     period <- NULL
   }
   if (hidNULL) {
     hid <- NULL
   }
-
+  if (clusterNULL){
+    cluster <- NULL
+  }
+  if (strataNULL){
+    strata <- NULL
+  }
+  if (pidNULL){
+    pid <- NULL
+  }
+  
   setattr(dat, "weights", weights)
   setattr(dat, "period", period)
   setattr(dat, "b.rep", w.names)
   setattr(dat, "hid", hid)
-
+  setattr(dat, "pid", pid)
+  setattr(dat, "cluster", cluster)
+  setattr(dat, "strata", strata)
+  setattr(dat, "totals", totals)
+  
+  
   return(dat)
 }
